@@ -79,6 +79,7 @@ class Bot:
         # gateway
         self._gateway_endpoint = None
         self._ws_conn = None
+        self._ws_close_code = None
         self._ws_reconnect = False
         
         # command
@@ -150,12 +151,12 @@ class Bot:
         async def check_death(dt):
             if self._die:
                 self._stop_periodics = True
+                self._ws_close_code = 1001
                 try:
-                    c = self._ws_conn
-                    self._ws_conn = None
-                    await c.close(1001) # not returning
+                    await self._ws_conn.close(self._ws_close_code)
                 except:
                     pass
+                self._ws_conn = None
         self.register_periodic(0.1, check_death)
         
         '''
@@ -314,11 +315,14 @@ class Bot:
                 try:
                     await util.call_func(f, dt)
                 except BaseException as e:
+                    # make sure it will not run for at least 5 seconds
+                    d[1] = time.time() + max(0, 5 - call_rate)
+                    
                     p_err = error.CallbackError("Periodic function '%s' errored." % repr(f))
                     p_err.__cause__ = e
                     self.log_error(p_err)
-                
-                d[1] = time.time()
+                else:
+                    d[1] = time.time()
             try:
                 await asyncio.sleep(0.01)
             except BaseException as e:
@@ -327,7 +331,12 @@ class Bot:
     
     async def _send_payload(self, pl):
         assert isinstance(pl, api.payload.OutgoingPayload), "payload must be discordbot.api.payload.OutgoingPayload"
-        await self._ws_conn.send(pl.dumps())
+        try:
+            await self._ws_conn.send(pl.dumps())
+        except websockets.ConnectionClosedError as e:
+            self._ws_close_code = self._ws_conn.close_code
+            self._ws_conn = None
+            raise e
         
     async def _wait_for_valid_payload(self):
         while True:
@@ -335,6 +344,10 @@ class Bot:
                 return api.payload.Payload(await self._ws_conn.recv())
             except error.ReceivedInvalidPayload as e:
                 self.log_error(e)
+            except websockets.ConnectionClosedError as e:
+                self._ws_close_code = self._ws_conn.close_code
+                self._ws_conn = None
+                raise e
     
     async def _wait_for_payload(self, opcode, wait_for_correct_opcode = False):
         while True:
@@ -467,7 +480,8 @@ class Bot:
         # close the previous websocket (if there is one)
         try:
             if self._ws_conn != None:
-                await self._ws_conn.close(1000)
+                self._ws_close_code = 1006
+                await self._ws_conn.close(1006) # will reconnect
         except:
             # it was already closed
             pass
@@ -481,14 +495,20 @@ class Bot:
         
         # identify or resume
         if self.session_id == None:
+            if self._ws_close_code != None:
+                # this is the second (or more) time that i've identified
+                # there must be a 1-5 second wait
+                await asyncio.sleep(3)
+            
             ready = None
             try:
                 ready = await self._ws_init_identify()
             except BaseException as e:
                 raise error.GatewayError("Failed to identify, perhaps invalid bot token?") from e
             
-            self.log("Running ready event after identify success.")
-            await self._handle_event(ready, event.ready)
+            if self._ws_close_code == None: # first connection
+                self.log("Running ready event after identify success.")
+                await self._handle_event(ready, event.ready)
         else:
             try:
                 await self._ws_init_resume()
@@ -537,22 +557,22 @@ class Bot:
                         break
             except BaseException as e:
                 if util.error_caused_by(e, websockets.ConnectionClosedError):
-                    if self._ws_conn:
-                        if self._ws_conn.close_code == 4004:
-                            p_err = error.DiscordError("Invalid bot token.")
-                            p_err.__cause__ = e
-                            self.log_error(p_err)
-                            return False
-                        elif self._ws_conn.close_code == 4009:
-                            self.session_id = None
-                        self.log("Bot will reconnect due to websocket closing with code %d." % self._ws_conn.close_code)
+                    if self._ws_close_code == 4004:
+                        p_err = error.DiscordError("Invalid bot token.")
+                        p_err.__cause__ = e
+                        self.log_error(p_err)
+                        return False
+                    elif self._ws_close_code == 4009:
+                        self.session_id = None
+                    else:
+                        self.log("Bot will reconnect due to websocket closing with code %d." % self._ws_close_code)
                     self._ws_reconnect = True
                 
                 if self._ws_reconnect:
                     continue
                 
+                self._die = True
                 if util.error_caused_by(e, (KeyboardInterrupt, asyncio.CancelledError)):
-                    self._die = True
                     self.log("Bot shutting down due to KeyboardInterrupt or asyncio.CancelledError.", severity = log_severity.info)
                     return True
                 
